@@ -14,6 +14,7 @@ from .eval import cfg_interpolate
 from .utils import env, expand_time
 from .tasks import TaskFile
 from .messages import encode_msg
+from .targets import sos_targets
 
 
 class TaskEngine(threading.Thread):
@@ -45,6 +46,7 @@ class TaskEngine(threading.Thread):
 
         self.task_status = OrderedDict()
         self.task_info = defaultdict(dict)
+        self.task_results = {}
         self.last_checked = None
         if 'status_check_interval' not in self.config:
             self.status_check_interval = 10
@@ -103,9 +105,12 @@ class TaskEngine(threading.Thread):
     def monitor_tasks(self, tasks=None, status=None, age=None):
         '''Start monitoring specified or all tasks'''
         self.engine_ready.wait()
+
         if not tasks:
             tasks = self.task_status.keys()
+            missing_tasks = []
         else:
+            missing_tasks = [x for x in tasks if x not in self.task_status]
             tasks = [x for x in tasks if x in self.task_status]
 
         # we only monitor running tasks
@@ -132,7 +137,9 @@ class TaskEngine(threading.Thread):
                  self.task_info[x].get('date',
                                        (time.time(), None, None))[0] < -age)))
         ],
-                      key=lambda x: -x[2][0])
+                      key=lambda x: -x[2][0]) + [
+                          (x, 'missing', '') for x in missing_tasks
+                      ]
 
     def get_tasks(self):
         with threading.Lock():
@@ -498,6 +505,7 @@ class TaskEngine(threading.Thread):
                     self.task_info[task_id]['date'] = [None, None, None]
                 if task_id in self.task_status and self.task_status[
                         task_id] == status:
+                    # nothing has changed.
                     self.notify_controller({
                         'queue': self.agent.alias,
                         'task_id': task_id,
@@ -548,12 +556,46 @@ class TaskEngine(threading.Thread):
                 if status in ('completed', 'failed'
                              ) and task_id in self.running_pending_tasks:
                     self.running_pending_tasks.pop(task_id)
+                # status changed to completed
+                self.task_results[task_id] = self._thread_workers.submit(
+                    self.agent.receive_result,
+                    task_id)
             # for running pending tasks
             if status == 'aborted' and task_id in self.running_pending_tasks:
                 self.pending_tasks.append(task_id)
                 self.task_status[task_id] = 'pending'
                 self.running_pending_tasks.pop(task_id)
 
+    def get_results(self, task_ids):
+        res = {}
+        while True:
+            with threading.Lock():
+                for task_id in task_ids:
+                    if task_id in self.task_results:
+                        if self.task_results[task_id].running():
+                            time.sleep(0.1)
+                        else:
+                            res[task_id] = self.task_results[task_id].result()
+                    elif task_id in self.task_status:
+                        if self.task_status[task_id] in ('running', 'pending', 'submitted'):
+                            time.sleep(0.1)
+                        else:
+                            res[task_id] = {
+                                'task': task_id,
+                                'exception': ValueError(f'Task {task_id} returns status {self.task_status[task_id]}'),
+                                'ret_code': 1,
+                                'output': sos_targets()
+                            }
+                    else:
+                        res[task_id] = {
+                                'task': task_id,
+                                'exception': ValueError(f'Missing task {task_id}'),
+                                'ret_code': 1,
+                                'output': sos_targets()
+                        }
+            if len(res) == len(task_ids):
+                return res
+
     def query_tasks(self,
                     tasks=None,
                     check_all=False,
@@ -673,6 +715,7 @@ class BackgroundProcess_TaskEngine(TaskEngine):
 
     def __init__(self, agent):
         super(BackgroundProcess_TaskEngine, self).__init__(agent)
+        self.wait_for_task = False
         if 'task_template' in self.config:
             self.task_template = self.config['task_template'].replace(
                 '\r\n', '\n')
