@@ -9,8 +9,7 @@ import shutil
 import socket
 import sys
 import time
-import unittest
-
+import platform
 import pytest
 
 from sos import execute_workflow
@@ -19,8 +18,8 @@ from sos.targets import file_target
 from sos.utils import env
 
 
-def internet_on(host='8.8.8.8', port=80, timeout=3):
-    '''Test if internet is connected '''
+def internet_on(host="8.8.8.8", port=80, timeout=3):
+    """Test if internet is connected """
     try:
         socket.setdefaulttimeout(timeout)
         socket.create_connection(("www.google.com", 80))
@@ -33,72 +32,104 @@ def internet_on(host='8.8.8.8', port=80, timeout=3):
 with_network = internet_on()
 
 
-def testAcceptableArgs():
-    '''test acceptable args of options'''
+def test_acceptable_args():
+    """test acceptable args of options"""
     with pytest.raises(Exception):
-        execute_workflow(r"""
+        execute_workflow(
+            r"""
             run: unrecog=1
             echo 'a'
-            """)
+            """
+        )
 
 
-def testGetOutput():
-    '''Test utility function get_output'''
-    execute_workflow(r"""
+def test_get_output():
+    """Test utility function get_output"""
+    execute_workflow(
+        r"""
         [0: shared='ret']
         ret = get_output('echo blah')
-        """)
+        """
+    )
     # use strip because there would be \r\n under windows
-    assert env.sos_dict['ret'].strip() == 'blah'
+    assert env.sos_dict["ret"].strip() == "blah"
 
-    execute_workflow(r"""
+    execute_workflow(
+        r"""
         [0: shared='ret']
         ret = get_output('echo blah', show_command=True)
-        """)
-    assert [x.strip() for x in env.sos_dict['ret'].splitlines()
-           ] == ['$ echo blah', 'blah']
+        """
+    )
+    assert [x.strip() for x in env.sos_dict["ret"].splitlines()] == [
+        "$ echo blah",
+        "blah",
+    ]
 
-    execute_workflow(r"""
+    execute_workflow(
+        r"""
         [0: shared='ret']
         ret = get_output('echo blah', show_command=True, prompt='% ')
-        """)
-    assert [x.strip() for x in env.sos_dict['ret'].splitlines()
-           ] == ['% echo blah', 'blah']
+        """
+    )
+    assert [x.strip() for x in env.sos_dict["ret"].splitlines()] == [
+        "% echo blah",
+        "blah",
+    ]
 
     with pytest.raises(Exception):
-        execute_workflow(r"""
+        execute_workflow(
+            r"""
             [0]
             get_output('catmouse')
-        """)
+        """
+        )
 
     with pytest.raises(Exception):
-        execute_workflow(r"""
+        execute_workflow(
+            r"""
             [0]
             ret = get_output('cat -h')
-            """)
+            """
+        )
 
 
-def testFailIf(temp_factory):
-    '''Test action fail if'''
-    temp_factory('a.txt')
+@pytest.mark.skipif(
+    not shutil.which("bash") or sys.platform == "win32", reason="Needs bash."
+)
+def test_get_output_extra_kwargs():
+    execute_workflow(
+        r"""
+        [0]
+        ret = get_output('echo "ECHO" | wc -l', executable="/bin/bash")
+        """
+    )
+
+
+def test_fail_if(temp_factory):
+    """Test action fail if"""
+    temp_factory("a.txt")
 
     # should fail in dryrun mode
     with pytest.raises(Exception):
-        execute_workflow(r"""
+        execute_workflow(
+            r"""
             [0]
             input: 'a.txt'
             fail_if(len(input) == 1)
-            """)
+            """
+        )
 
     with pytest.raises(Exception):
-        execute_workflow(r"""
+        execute_workflow(
+            r"""
             [0]
             input: 'a.txt', 'b.txt'
             fail_if(len(input) == 2)
-        """)
+        """
+        )
 
 
-def testDelayedFailIf():
+def test_delayed_fail_if():
     # test fail_if of killing another running substep
     st = time.time()
     with pytest.raises(Exception):
@@ -114,13 +145,15 @@ def testDelayedFailIf():
             time.sleep(2)
             fail_if(True)
             """,
-            config={'worker_procs': ['3']})
+            config={"worker_procs": ["3"]},
+        )
 
-    assert time.time(
-    ) - st >= 8, 'Test test should fail only after step 10 is completed'
+    assert (
+        time.time() - st >= 8
+    ), "Test test should fail only after step 10 is completed"
 
 
-def testDelayedFailIfFromNestedWorkflow():
+def test_delayed_fail_if_from_nested_workflow():
     # test fail_if of killing another running substep
 
     st = time.time()
@@ -140,34 +173,39 @@ def testDelayedFailIfFromNestedWorkflow():
             time.sleep(2)
             fail_if(True)
         """,
-            config={'worker_procs': ['3']})
+            config={"worker_procs": ["3"]},
+        )
 
-    assert time.time(
-    ) - st >= 8, 'Test test should fail only after step 10 is completed'
+    assert (
+        time.time() - st >= 8
+    ), "Test test should fail only after step 10 is completed"
 
 
-def testWarnIf(temp_factory):
-    '''Test action fail if'''
+def test_warn_if(temp_factory):
+    """Test action fail if"""
     execute_workflow(
         r"""
         [0]
         warn_if(input is None, 'Expect to see a warning message')
         """,
-        options={'run_mode': 'dryrun'})
+        options={"run_mode": "dryrun"},
+    )
 
-    temp_factory('a.txt', 'b.txt')
+    temp_factory("a.txt", "b.txt")
     execute_workflow(
         r"""
         [0]
         input: 'a.txt', 'b.txt'
         warn_if(len(_input) == 1)
         """,
-        options={'run_mode': 'dryrun'})
+        options={"run_mode": "dryrun"},
+    )
 
 
-def testStopIf():
-    '''Test action stop_if'''
-    execute_workflow(r'''
+def test_stop_if():
+    """Test action stop_if"""
+    execute_workflow(
+        r"""
         [0: shared='result']
         rep = range(20)
         result = []
@@ -175,16 +213,18 @@ def testStopIf():
 
         stop_if(_rep > 10)
         result.append(_rep)
-        ''')
-    assert env.sos_dict['result'] == [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
+        """
+    )
+    assert env.sos_dict["result"] == [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
 
 
 def test_stop_if_1(clear_now_and_after):
     # stop_if should not be treated as error so the previously
     # generated output file will be removed
-    clear_now_and_after('test_stop_if_0.txt', 'test_stop_if_1.txt')
+    clear_now_and_after("test_stop_if_0.txt", "test_stop_if_1.txt")
 
-    execute_workflow(r'''
+    execute_workflow(
+        r"""
         [10]
         rep = range(2)
         input: for_each='rep'
@@ -196,18 +236,20 @@ def test_stop_if_1(clear_now_and_after):
         [20]
         assert(step_input.contains('test_stop_if_0.txt'))
         assert(not step_input.contains('test_stop_if_1.txt'))
-        ''')
+        """
+    )
 
-    assert os.path.isfile('test_stop_if_0.txt')
-    assert not (os.path.isfile('test_stop_if_1.txt'))
+    assert os.path.isfile("test_stop_if_0.txt")
+    assert not (os.path.isfile("test_stop_if_1.txt"))
 
 
 def test_stop_if_2(clear_now_and_after):
     # stop_if should not be treated as error so the previously
     # generated output file will not be removed
-    clear_now_and_after('test_stop_if_0.txt', 'test_stop_if_1.txt')
+    clear_now_and_after("test_stop_if_0.txt", "test_stop_if_1.txt")
 
-    execute_workflow(r'''
+    execute_workflow(
+        r"""
         [10]
         rep = range(2)
         input: for_each='rep'
@@ -219,14 +261,16 @@ def test_stop_if_2(clear_now_and_after):
         [20]
         assert(step_input.contains('test_stop_if_0.txt'))
         assert(step_input.contains('test_stop_if_1.txt'))
-        ''')
-    assert os.path.isfile('test_stop_if_0.txt')
-    assert os.path.isfile('test_stop_if_1.txt')
+        """
+    )
+    assert os.path.isfile("test_stop_if_0.txt")
+    assert os.path.isfile("test_stop_if_1.txt")
 
 
-def testSkipIf():
-    '''Test action stop_if'''
-    execute_workflow(r'''
+def test_skip_if():
+    """Test action stop_if"""
+    execute_workflow(
+        r"""
         [0: shared='result']
         rep = range(20)
         result = []
@@ -234,17 +278,19 @@ def testSkipIf():
 
         skip_if(_rep > 10)
         result.append(_rep)
-        ''')
-    assert env.sos_dict['result'] == [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
+        """
+    )
+    assert env.sos_dict["result"] == [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
 
 
 def test_skip_if_1(clear_now_and_after):
-    clear_now_and_after([f'test_stop_if_{rep}.txt' for rep in range(2)])
+    clear_now_and_after([f"test_stop_if_{rep}.txt" for rep in range(2)])
 
     # stop_if should not be treated as error so the previously
     # generated output file will be removed
 
-    execute_workflow(r'''
+    execute_workflow(
+        r"""
         [10]
         rep = range(2)
         input: for_each='rep'
@@ -257,17 +303,19 @@ def test_skip_if_1(clear_now_and_after):
         assert(step_input.contains('test_stop_if_0.txt'))
         assert(not step_input.contains('test_stop_if_1.txt'))
 
-        ''')
+        """
+    )
 
-    assert os.path.isfile('test_stop_if_0.txt')
-    assert not os.path.isfile('test_stop_if_1.txt')
+    assert os.path.isfile("test_stop_if_0.txt")
+    assert not os.path.isfile("test_stop_if_1.txt")
 
 
-def testDoneIf(clear_now_and_after):
-    'Test action done_if'
-    clear_now_and_after([f'test_done_if_{rep}.txt' for rep in range(2)])
+def test_done_if(clear_now_and_after):
+    "Test action done_if"
+    clear_now_and_after([f"test_done_if_{rep}.txt" for rep in range(2)])
 
-    execute_workflow(r'''
+    execute_workflow(
+        r"""
         [10]
         rep = range(2)
         input: for_each='rep'
@@ -280,58 +328,69 @@ def testDoneIf(clear_now_and_after):
         assert(step_input.contains('test_done_if_0.txt'))
         assert(step_input.contains('test_done_if_1.txt'))
 
-        ''')
-    assert (os.path.isfile('test_done_if_0.txt'))
-    assert (os.path.isfile('test_done_if_1.txt'))
+        """
+    )
+    assert os.path.isfile("test_done_if_0.txt")
+    assert os.path.isfile("test_done_if_1.txt")
 
 
-def testRun():
-    '''Test action run'''
-    execute_workflow(r'''
+def test_run():
+    """Test action run"""
+    execute_workflow(
+        r"""
         [0]
         run:
         echo 'Echo'
-        ''')
+        """
+    )
 
 
 @pytest.mark.skipif(
-    sys.platform == 'win32', reason="Under windows, echo 'Echo is perfectly OK")
+    sys.platform == "win32", reason="Under windows, echo 'Echo is perfectly OK"
+)
 def test_run_1():
 
     with pytest.raises(Exception):
-        execute_workflow(r'''
+        execute_workflow(
+            r"""
             [0]
             run:
             echo 'Echo
-            ''')
+            """
+        )
 
 
-def testRunWithShebang():
-    execute_workflow(r'''
+def test_run_with_shebang():
+    execute_workflow(
+        r"""
         [0]
         run:
         #!/usr/bin/env python
         print('Echo')
-        ''')
+        """
+    )
 
 
-@pytest.mark.skipif(not shutil.which('perl'), reason="Needs perl")
-def testPerl():
-    '''Test action perl'''
-    execute_workflow(r'''
+@pytest.mark.skipif(not shutil.which("perl"), reason="Needs perl")
+def test_perl():
+    """Test action perl"""
+    execute_workflow(
+        r"""
         [0]
         perl:
         use strict;
         use warnings;
 
         print "hi NAME\n";
-        ''')
+        """
+    )
 
 
-@pytest.mark.skipif(not shutil.which('ruby'), reason="Needs ruby")
-def testRuby():
-    '''Test action ruby'''
-    execute_workflow(r'''
+@pytest.mark.skipif(not shutil.which("ruby"), reason="Needs ruby")
+def test_ruby():
+    """Test action ruby"""
+    execute_workflow(
+        r"""
         [0]
         ruby:
         line1 = "Cats are smarter than dogs";
@@ -343,100 +402,117 @@ def testRuby():
         if ( line2 =~ /Cats(.*)/ )
         puts "Line2 contains  Dogs"
         end
-        ''')
+        """
+    )
 
 
 @pytest.mark.skipif(
-    not with_network or 'TRAVIS' in os.environ or 'APPVEYOR' in os.environ,
-    reason='Skip test because of no internet connection or in travis test')
-def testDownload(temp_factory, clear_now_and_after):
-    '''Test download of resources'''
+    not with_network or "TRAVIS" in os.environ or "APPVEYOR" in os.environ,
+    reason="Skip test because of no internet connection or in travis test",
+)
+def test_download(temp_factory, clear_now_and_after):
+    """Test download of resources"""
 
-    clear_now_and_after('tmp')
-    temp_factory(dir='tmp')
+    clear_now_and_after("tmp")
+    temp_factory(dir="tmp")
 
     # test decompress tar.gz file
-    execute_workflow(r'''
+    execute_workflow(
+        r"""
         [0]
         download(['http://bioinformatics.mdanderson.org/Software/VariantTools/repository/annoDB/CancerGeneCensus-20170912.DB.gz'],
         dest_dir='tmp', decompress=True)
-        ''')
-    assert (os.path.isfile('tmp/CancerGeneCensus-20170912.DB'))
+        """
+    )
+    assert os.path.isfile("tmp/CancerGeneCensus-20170912.DB")
 
     #
     # testing the download of single file
     #
-    execute_workflow(r'''
+    execute_workflow(
+        r"""
         [0]
         download: dest_file='tmp/refgene.ppp'
         http://bioinformatics.mdanderson.org/Software/VariantTools/repository/resource/refgene.pkl
-        ''')
+        """
+    )
 
-    assert (os.path.isfile('tmp/refgene.ppp'))
+    assert os.path.isfile("tmp/refgene.ppp")
 
     # test option dest_dir
-    execute_workflow(r'''
+    execute_workflow(
+        r"""
         [0]
         download: dest_dir='tmp'
         http://bioinformatics.mdanderson.org/Software/VariantTools/repository/resource/refgene.pkl
-        ''')
-    assert (os.path.isfile('tmp/refgene.pkl'))
+        """
+    )
+    assert os.path.isfile("tmp/refgene.pkl")
 
 
 @pytest.mark.skipif(
-    not with_network or 'TRAVIS' in os.environ or 'APPVEYOR' in os.environ,
-    reason='Skip test because of no internet connection or in travis test')
-def testDownloadMissingFile(temp_factory, clear_now_and_after):
+    not with_network or "TRAVIS" in os.environ or "APPVEYOR" in os.environ,
+    reason="Skip test because of no internet connection or in travis test",
+)
+def test_download_missing_file(temp_factory, clear_now_and_after):
 
-    clear_now_and_after('tmp')
-    temp_factory(dir='tmp')
+    clear_now_and_after("tmp")
+    temp_factory(dir="tmp")
 
     with pytest.raises(Exception):
-        execute_workflow(r'''
+        execute_workflow(
+            r"""
             [0]
             download: dest_dir='tmp', decompress=True, max_jobs=2
             http://bioinformatics.mdanderson.org/Software/VariantTools/repository/resource/non-existing.gz
             http://bioinformatics.mdanderson.org/Software/VariantTools/repository/annoDB/CancerGeneCensus-20170912.DB.gz
             http://bioinformatics.mdanderson.org/Software/VariantTools/repository/annoDB/CancerGeneCensus.ann
             http://bioinformatics.mdanderson.org/Software/VariantTools/repository/annoDB/DGV-hg38_20160831.ann
-            ''')
+            """
+        )
 
-    assert (os.path.isfile('tmp/CancerGeneCensus.ann'))
+    assert os.path.isfile("tmp/CancerGeneCensus.ann")
 
 
 @pytest.mark.skipif(
-    not with_network or 'TRAVIS' in os.environ or 'APPVEYOR' in os.environ,
-    reason='Skip test because of no internet connection or in travis test')
-def testDownloadLargeFile(temp_factory, clear_now_and_after):
+    not with_network or "TRAVIS" in os.environ or "APPVEYOR" in os.environ,
+    reason="Skip test because of no internet connection or in travis test",
+)
+def test_download_large_file(temp_factory, clear_now_and_after):
     # test decompress tar.gz, .zip and .gz files
 
-    clear_now_and_after('tmp')
-    temp_factory(dir='tmp')
+    clear_now_and_after("tmp")
+    temp_factory(dir="tmp")
 
-    execute_workflow(r'''
+    execute_workflow(
+        r"""
         [0]
         download: dest_dir='tmp', decompress=True
         http://bioinformatics.mdanderson.org/Software/VariantTools/repository/annoDB/CancerGeneCensus-20170912.DB.gz
         http://bioinformatics.mdanderson.org/Software/VariantTools/repository/annoDB/CancerGeneCensus.ann
         http://bioinformatics.mdanderson.org/Software/VariantTools/repository/annoDB/DGV-hg38_20160831.ann
-        ''')
+        """
+    )
 
     # run in build mode
-    execute_workflow(r'''
+    execute_workflow(
+        r"""
         [0]
         download: dest_dir='tmp', decompress=True
         http://bioinformatics.mdanderson.org/Software/VariantTools/repository/annoDB/CancerGeneCensus-20170912.DB.gz
         http://bioinformatics.mdanderson.org/Software/VariantTools/repository/annoDB/CancerGeneCensus.ann
         http://bioinformatics.mdanderson.org/Software/VariantTools/repository/annoDB/DGV-hg38_20160831.ann
-        ''')
+        """
+    )
 
 
-@pytest.mark.skipif(not shutil.which('pandoc'), reason="Needs pandoc")
-def testPandoc(clear_now_and_after):
-    '''Test action pandoc'''
-    clear_now_and_after('report.md', 'myreport.html')
+@pytest.mark.skipif(not shutil.which("pandoc"), reason="Needs pandoc")
+def test_pandoc(clear_now_and_after):
+    """Test action pandoc"""
+    clear_now_and_after("report.md", "myreport.html")
 
-    execute_workflow(r'''
+    execute_workflow(
+        r"""
         [10]
 
         report: output='report.md'
@@ -449,16 +525,18 @@ def testPandoc(clear_now_and_after):
         # generate report
         output: 'myreport.html'
         pandoc(input='report.md', output=_output[0])
-        ''')
+        """
+    )
 
-    assert (os.path.isfile('myreport.html'))
+    assert os.path.isfile("myreport.html")
 
 
-@pytest.mark.skipif(not shutil.which('pandoc'), reason="Needs pandoc")
+@pytest.mark.skipif(not shutil.which("pandoc"), reason="Needs pandoc")
 def test_pandoc_1(clear_now_and_after):
-    clear_now_and_after('a.md', 'myreport.html')
+    clear_now_and_after("a.md", "myreport.html")
 
-    execute_workflow(r'''
+    execute_workflow(
+        r"""
         [10]
         report: output='a.md'
         ## Some random figure
@@ -470,17 +548,19 @@ def test_pandoc_1(clear_now_and_after):
         # generate report
         output: 'myreport.html'
         pandoc(input='a.md', output=_output[0])
-        ''')
+        """
+    )
 
-    assert os.path.isfile('myreport.html')
+    assert os.path.isfile("myreport.html")
 
 
-@pytest.mark.skipif(not shutil.which('pandoc'), reason="Needs pandoc")
+@pytest.mark.skipif(not shutil.which("pandoc"), reason="Needs pandoc")
 def test_pandoc_2(clear_now_and_after):
-    clear_now_and_after('a.md', 'myreport.html')
+    clear_now_and_after("a.md", "myreport.html")
     #
     # another case is no output
-    execute_workflow(r'''
+    execute_workflow(
+        r"""
         [10]
         report: output='a.md'
         ## Some random figure
@@ -491,15 +571,17 @@ def test_pandoc_2(clear_now_and_after):
         [100]
         # generate report
         pandoc(input='a.md')
-        ''')
+        """
+    )
 
 
-@pytest.mark.skipif(not shutil.which('pandoc'), reason="Needs pandoc")
+@pytest.mark.skipif(not shutil.which("pandoc"), reason="Needs pandoc")
 def test_pandoc_3(clear_now_and_after):
     # test acceptance of a list of input filenames
-    clear_now_and_after('default_10.md', 'default_20.md', 'output.html')
+    clear_now_and_after("default_10.md", "default_20.md", "output.html")
 
-    execute_workflow(r'''
+    execute_workflow(
+        r"""
         [10]
         report: output='default_10.md'
         A_10
@@ -511,36 +593,38 @@ def test_pandoc_3(clear_now_and_after):
         [100]
         # generate report
         pandoc(input=['default_10.md', 'default_20.md'], output='output.html')
-        ''')
+        """
+    )
 
-    for f in ['default_10.md', 'default_20.md', 'output.html']:
+    for f in ["default_10.md", "default_20.md", "output.html"]:
         assert file_target(f).target_exists()
 
 
-def testReport(clear_now_and_after):
-    '''Test action report'''
-    clear_now_and_after('report.txt')
+def test_report(clear_now_and_after):
+    """Test action report"""
+    clear_now_and_after("report.txt")
 
-    script = r'''
+    script = r"""
         [A]
         parameter: num=5
         report: output='report.txt', expand=True
         touch {num}.txt
 
-        '''
+        """
     # run twice
-    execute_workflow(script, args=['--num', '7'])
-    execute_workflow(script, args=['--num', '5'])
+    execute_workflow(script, args=["--num", "7"])
+    execute_workflow(script, args=["--num", "5"])
 
-    with open('report.txt') as report:
-        assert report.read() == 'touch 5.txt\n\n'
+    with open("report.txt") as report:
+        assert report.read() == "touch 5.txt\n\n"
 
 
 def test_report_1(clear_now_and_after):
-    '''Test action report'''
-    clear_now_and_after('report.txt')
+    """Test action report"""
+    clear_now_and_after("report.txt")
 
-    execute_workflow(r'''
+    execute_workflow(
+        r"""
         [A]
         report: output='report.txt', expand=True
         {step_name}
@@ -548,33 +632,37 @@ def test_report_1(clear_now_and_after):
         [A_10]
         report: output='report.txt', expand=True
         {step_name}
-        ''')
-    with open('report.txt') as report:
-        assert report.read() == 'A_10\n\n'
+        """
+    )
+    with open("report.txt") as report:
+        assert report.read() == "A_10\n\n"
 
 
 def test_report_2(clear_now_and_after):
-    '''Test action report'''
-    clear_now_and_after('a.txt', 'out.txt')
+    """Test action report"""
+    clear_now_and_after("a.txt", "out.txt")
 
-    execute_workflow(r'''
+    execute_workflow(
+        r"""
         [A_1]
         run: output='a.txt'
             echo something > a.txt
 
         report(input='a.txt', output='out.txt')
-        ''')
+        """
+    )
 
-    for name in ('a.txt', 'out.txt'):
+    for name in ("a.txt", "out.txt"):
         with open(name) as report:
-            assert report.read().strip() == 'something'
+            assert report.read().strip() == "something"
 
 
 def test_report_3(clear_now_and_after):
-    '''Test action report'''
-    clear_now_and_after('report.txt')
+    """Test action report"""
+    clear_now_and_after("report.txt")
     #
-    execute_workflow(r'''
+    execute_workflow(
+        r"""
         [A_1]
         run: output='a.txt'
         echo something > a.txt
@@ -585,98 +673,111 @@ def test_report_3(clear_now_and_after):
 
         [A_3]
         report(input=['a.txt', 'b.txt'], output='out.txt')
-        ''')
-    for name in ('a.txt', 'b.txt', 'out.txt'):
-        assert (file_target(name).target_exists())
+        """
+    )
+    for name in ("a.txt", "b.txt", "out.txt"):
+        assert file_target(name).target_exists()
 
 
 def test_report_4(clear_now_and_after):
-    '''Test action report'''
-    clear_now_and_after('a.txt')
+    """Test action report"""
+    clear_now_and_after("a.txt")
 
     # test report to other types of output: path
-    execute_workflow(r'''
+    execute_workflow(
+        r"""
         [A_1]
         report: output=path('a.txt')
         something
-        ''')
+        """
+    )
 
 
 def test_report_5(clear_now_and_after):
-    '''Test action report'''
-    clear_now_and_after('a.txt')
+    """Test action report"""
+    clear_now_and_after("a.txt")
 
-    execute_workflow(r'''
+    execute_workflow(
+        r"""
         [A_1]
         output: 'a.txt'
         report: output=_output[0]
         something
-        ''')
+        """
+    )
 
 
 def test_report_6(clear_now_and_after):
-    '''Test action report'''
-    clear_now_and_after('a.txt')
+    """Test action report"""
+    clear_now_and_after("a.txt")
 
     # test report to other types of output: sos_targets
-    execute_workflow(r'''
+    execute_workflow(
+        r"""
         [A_1]
         output: 'a.txt'
         report: output=_output
         something
-        ''')
+        """
+    )
 
 
 def test_report_7(clear_now_and_after):
-    '''Test action report'''
-    clear_now_and_after('a.txt')
+    """Test action report"""
+    clear_now_and_after("a.txt")
 
     with pytest.raises(Exception):
-        execute_workflow(r'''
+        execute_workflow(
+            r"""
             [A_1]
             output: 'a.txt', 'b.txt'
             report: output=_output
             something
-            ''')
+            """
+        )
 
 
-def testOptionWorkdir(temp_factory):
-    '''Test option workdir of tasks'''
-    temp_factory(dir='temp_wdr')
+def test_option_workdir(temp_factory):
+    """Test option workdir of tasks"""
+    temp_factory(dir="temp_wdr")
 
-    with open(os.path.join('temp_wdr', 'a.txt'), 'w') as tmp:
-        tmp.write('hello')
+    with open(os.path.join("temp_wdr", "a.txt"), "w") as tmp:
+        tmp.write("hello")
 
-    execute_workflow(r'''
+    execute_workflow(
+        r"""
         [A_1]
         run: workdir='temp_wdr'
         cp -f a.txt a2.txt
-        ''')
+        """
+    )
 
-    assert file_target(os.path.join('temp_wdr', 'a2.txt')).target_exists()
-    with open(os.path.join('temp_wdr', 'a.txt')) as tmp:
-        assert 'hello' == tmp.read()
+    assert file_target(os.path.join("temp_wdr", "a2.txt")).target_exists()
+    with open(os.path.join("temp_wdr", "a.txt")) as tmp:
+        assert "hello" == tmp.read()
 
 
-def testActionScript():
-    '''Test action script'''
-    execute_workflow(r'''
+def test_action_script():
+    """Test action script"""
+    execute_workflow(
+        r"""
         [A_1]
         script: interpreter='python'
         with open('something.txt', 'w') as tmp:
             tmp.write('something')
-        ''')
+        """
+    )
 
-    assert file_target('something.txt').target_exists()
-    with open('something.txt') as tmp:
-        assert 'something' == tmp.read()
+    assert file_target("something.txt").target_exists()
+    with open("something.txt") as tmp:
+        assert "something" == tmp.read()
 
 
-def testRegenerateReport(clear_now_and_after):
-    '''Testing the regeneration of report once is needed. The problem
-    here is the 'input' parameter of report.'''
-    clear_now_and_after('a1.md', 'a2.md', 'out.md')
-    script = r'''
+def test_regenerate_report(clear_now_and_after):
+    """Testing the regeneration of report once is needed. The problem
+    here is the 'input' parameter of report."""
+    clear_now_and_after("a1.md", "a2.md", "out.md")
+    script = r"""
         [A_1]
         output: 'a1.txt', 'a1.md'
         run:
@@ -696,24 +797,25 @@ def testRegenerateReport(clear_now_and_after):
         input: 'a1.md', 'a2.md'
         output: 'out.md'
         report:     input=['a1.md', 'a2.md'], output='out.md'
-        '''
+        """
     execute_workflow(script)
-    with open('a1.md') as a:
-        assert a.read() == 'a1\n\n'
-    with open('a2.md') as a:
-        assert a.read() == 'a2\n\n'
-    with open('out.md') as a:
-        assert a.read() == 'a1\n\na2\n\n'
+    with open("a1.md") as a:
+        assert a.read() == "a1\n\n"
+    with open("a2.md") as a:
+        assert a.read() == "a2\n\n"
+    with open("out.md") as a:
+        assert a.read() == "a1\n\na2\n\n"
 
-    clear_now_and_after('a1.md', 'a2.md', 'out.md')
+    clear_now_and_after("a1.md", "a2.md", "out.md")
     execute_workflow(script)
 
 
-def testActiveActionOption(temp_factory):
-    '''Test the active option of actions'''
+def test_active_action_option(temp_factory):
+    """Test the active option of actions"""
     # disallow
     with pytest.raises(Exception):
-        SoS_Script('''
+        SoS_Script(
+            """
 [1]
 rep = range(5)
 input: for_each = 'rep'
@@ -722,27 +824,29 @@ ff = f"{_rep}.txt"
 run:  expand=True, active=1,2
 echo {ff}
 touch temp/{ff}
-''')
+"""
+        )
     #
     for active, result in [
-        ('0', ['temp/0.txt']),
-        ('-1', ['temp/4.txt']),
-        ('(1,2)', ['temp/1.txt', 'temp/2.txt']),
-        ('[2,3]', ['temp/2.txt', 'temp/3.txt']),
-        ('(0,2,4)', ['temp/0.txt', 'temp/2.txt', 'temp/4.txt']),
-        ('slice(1,None)',
-         ['temp/1.txt', 'temp/2.txt', 'temp/3.txt', 'temp/4.txt']),
-        ('slice(1,-2)', ['temp/1.txt', 'temp/2.txt']),
-        ('slice(None,None,2)', ['temp/0.txt', 'temp/2.txt', 'temp/4.txt']),
-        ('True',
-         ['temp/0.txt', 'temp/1.txt', 'temp/2.txt', 'temp/3.txt',
-          'temp/4.txt']),
-        ('False', []),
+        ("0", ["temp/0.txt"]),
+        ("-1", ["temp/4.txt"]),
+        ("(1,2)", ["temp/1.txt", "temp/2.txt"]),
+        ("[2,3]", ["temp/2.txt", "temp/3.txt"]),
+        ("(0,2,4)", ["temp/0.txt", "temp/2.txt", "temp/4.txt"]),
+        ("slice(1,None)", ["temp/1.txt", "temp/2.txt", "temp/3.txt", "temp/4.txt"]),
+        ("slice(1,-2)", ["temp/1.txt", "temp/2.txt"]),
+        ("slice(None,None,2)", ["temp/0.txt", "temp/2.txt", "temp/4.txt"]),
+        (
+            "True",
+            ["temp/0.txt", "temp/1.txt", "temp/2.txt", "temp/3.txt", "temp/4.txt"],
+        ),
+        ("False", []),
     ]:
-        temp_factory(dir='temp')
+        temp_factory(dir="temp")
         # test first iteration
         execute_workflow(
-            ('''
+            (
+                """
             [1]
             rep = range(5)
             input: for_each = 'rep'
@@ -751,38 +855,42 @@ touch temp/{ff}
             run:  expand=True, active=%s
             echo {ff}
             touch temp/{ff}
-            ''' % active).replace('/', os.sep),
-            options={'sig_mode': 'force'})
+            """
+                % active
+            ).replace("/", os.sep),
+            options={"sig_mode": "force"},
+        )
 
-        files = list(glob.glob(os.path.join('temp', '*.txt')))
-        assert sorted(files) == sorted([x.replace('/', os.sep) for x in result])
+        files = list(glob.glob(os.path.join("temp", "*.txt")))
+        assert sorted(files) == sorted([x.replace("/", os.sep) for x in result])
 
 
 def test_action_option_template(clear_now_and_after):
-    clear_now_and_after('template_output.txt')
+    clear_now_and_after("template_output.txt")
 
-    execute_workflow('''
+    execute_workflow(
+        """
         run:  template='cat {filename}'
             echo 'whatever' > template_output.txt
-        ''')
-    assert not os.path.isfile('template_output.txt')
+        """
+    )
+    assert not os.path.isfile("template_output.txt")
 
 
 def test_action_option_template_name(config_factory):
-    cfg = config_factory('''\
+    cfg = config_factory(
+        """\
         action_templates:
             cat: |
                 cat {filename}
 
-        ''')
+        """
+    )
     execute_workflow(
-        '''
+        """
         run:  template_name='cat'
             echo 'whatever' > template_output.txt
-        ''',
-        args=['-c', cfg])
-    assert not os.path.isfile('template_output.txt')
-
-
-if __name__ == '__main__':
-    unittest.main()
+        """,
+        args=["-c", cfg],
+    )
+    assert not os.path.isfile("template_output.txt")
